var awsIot = require('aws-iot-device-sdk');
var environment = require("./constants.js");
var logLevel = process.env.consoleLogLevel || environment.consoleLogLevel;
var mqttLevel = process.env.mqttLogLevel || environment.mqttLogLevel;
var mqttTopic = environment.mqttTopic;
var map = {
    "off":6,
    "debug":1,
    "info":2,
    "warn":3,
    "error":4,
    "fatal":5
};
class Logger {
    debug(message) {
        var path = _getCallerFile();
        message = "[debug] " + path + " " + message;
        if(map["debug"] >= map[logLevel]){
            console.log(message);
        }
        if(map["debug"] >= map[mqttLevel]){
            checkMqttConnection(message);
        }
    }
    warn(message) {
        var path = _getCallerFile();
        message = "[warn] " + path + " " + message;
        if(map["warn"] >= map[logLevel]){
            console.log(message);
        }
        if(map["warn"] >= map[mqttLevel]){
            checkMqttConnection(message);
        }
    }
    info(message) {
        var path = _getCallerFile();
        message = "[info] " + path + " " + message
        if(map["info"] >= map[logLevel]){
            console.log(message);
        }
        if(map["info"] >= map[mqttLevel]){
            checkMqttConnection(message);
        }
    }
    error(message) {
        var path = _getCallerFile();
        message = "[error] " + path + " " + message;
        if(map["error"] >= map[logLevel]){         
            console.log(message);
        }
        if(map["error"] >= map[mqttLevel]){
            checkMqttConnection(message);
        }
    }
    fatal(message) {
        var path = _getCallerFile();
        message = "[fatal] " + path + " " + message
        if(map["fatal"] >= map[logLevel]){
            console.log(message);
        }
        if(map["fatal"] >= map[mqttLevel]){
            checkMqttConnection(message);
        }
    }
}

function _getCallerFile() {
    var originalFunc = Error.prepareStackTrace;

    var callerfile;
    try {
        var err = new Error();
        var currentfile;

        Error.prepareStackTrace = function (err, stack) { return stack; };

        currentfile = err.stack.shift().getFileName();

        while (err.stack.length) {
            callerfile = err.stack.shift().getFileName();

            if (currentfile !== callerfile) break;
        }
    } catch (e) { }

    Error.prepareStackTrace = originalFunc;

    return callerfile;
}

function checkMqttConnection(message){
    var certDetails = environment.awsCertDetails;
    var keyDetails = environment.awsKeys;
    if(!(certDetails.certPath.length <= 0 || certDetails.host.length <=0 || certDetails.privateKeyPath.length <= 0 || certDetails.rootCaPath.length <= 0)){
        var awsConnection = awsIot.device({
            keyPath: certDetails.privateKeyPath,
            certPath: certDetails.certPath+".crt",
            caPath: certDetails.rootCaPath+".crt",
            host: certDetails.host
        });
        publishDataOnMqtt(awsConnection,message);
    }
    else if(!(keyDetails.region.length <= 0 || keyDetails.accessKey.length <= 0 || keyDetails.secretKey.length <= 0 || keyDetails.host.length <= 0)){
        var awsConnection = awsIot.device({
            region: keyDetails.region,
            protocol: 'wss',
            accessKeyId: keyDetails.accessKey,
            secretKey: keyDetails.secretKey,
            host: keyDetails.host
        });
        publishDataOnMqtt(awsConnection,message);
    }

}

function publishDataOnMqtt(awsConnection,message){
    awsConnection.on('connect', function () {
        awsConnection.publish(mqttTopic, JSON.stringify({"message":message}),{},function(err){
            if(err){
                console.log(err);
            }
            awsConnection.end();
        });  
    });
    awsConnection.on('error', function (err) {
        if(err){
            console.log("Error",err);
        }
        awsConnection.end();
    });
}

module.exports = new Logger();